'use strict';

var _ = require('lodash');
var loadUserLevel = require('./loadUserLevel');
var async=require('async');
var request = require('request');

var env = process.env.NODE_ENV;
var API_ADDRESS = 'http://rest.wxpai.cn';
if (env == 'sandbox') {
    API_ADDRESS = 'http://rest.wxpai.cn';
} else if (env == 'production') {
    API_ADDRESS = 'https://rest.wxpai.cn';
}

exports.removeHTMLTag = function(str) {
    str = str.replace(/<\/?[^>]*>/g,''); //去除HTML tag
    str = str.replace(/[ | ]*\n/g,'\n'); //去除行尾空白
    str=str.replace(/&nbsp;/ig,'');//去掉&nbsp;
    return str;
}

exports.loadLevel=function(ent_code,items,callback){
    var openIds = [];
    _.forEach(items, function(d, i) {
        if(d.from && d.from.uid){
            openIds.push(d.from.uid);
        }
        if(d.to && d.to.uid){
            openIds.push(d.to.uid);
        }
        if(d.commentLevel1From && d.commentLevel1From.uid){
            openIds.push(d.commentLevel1From.uid);
        }
        if(d.commentLevel2From && d.commentLevel2From.uid){
            openIds.push(d.commentLevel2From.uid);
        }
        if(d.commentLevel2ThreadFrom && d.commentLevel2ThreadFrom.uid){
            openIds.push(d.commentLevel2ThreadFrom.uid);
        }
    });

    var tasks = [];
    _.forEach(openIds, function(open_id) {
        tasks.push(function(cont){
            loadLevelFromAPI(ent_code,open_id,cont);
        });
    });
    async.parallel(tasks,function(err,results){
        if (results && results.length>0) {//results: [{'egwegweg':1},{'gergergerg':3}]
            _.forEach(items, function(d, i) {
                if (items[i].toObject) {
                    items[i] = items[i].toObject();
                }
                _.forEach(results, function(r, j) {
                    if(r){
                        if(d.from && d.from.uid && r[d.from.uid]){
                            items[i].from.exp = r[d.from.uid];
                        }
                        if(d.to && d.to.uid && r[d.to.uid]){
                            items[i].to.exp = r[d.to.uid];
                        }
                        if(d.commentLevel1From && d.commentLevel1From.uid && r[d.commentLevel1From.uid]){
                            items[i].commentLevel1From.exp = r[d.commentLevel1From.uid];
                        }
                        if(d.commentLevel2From && d.commentLevel2From.uid && r[d.commentLevel2From.uid]){
                            items[i].commentLevel2From.exp = r[d.commentLevel2From.uid];
                        }
                        if(d.commentLevel2ThreadFrom && d.commentLevel2ThreadFrom.uid && r[d.commentLevel2ThreadFrom.uid]){
                            items[i].commentLevel2ThreadFrom.exp = r[d.commentLevel2ThreadFrom.uid];
                        }
                    }
                });

                if(d.from && !items[i].from.exp){
                    items[i].from.exp = 0;
                }
                if(d.to && !items[i].to.exp){
                    items[i].to.exp = 0;
                }
                if(d.commentLevel1From && !items[i].commentLevel1From.exp){
                    items[i].commentLevel1From.exp = 0;
                }
                if(d.commentLevel2From && !items[i].commentLevel2From.exp){
                    items[i].commentLevel2From.exp = 0;
                }
                if(d.commentLevel2ThreadFrom && !items[i].commentLevel2ThreadFrom.exp){
                    items[i].commentLevel2ThreadFrom.exp = 0;
                }
            });
        }
        return callback && callback();
    });
};

exports.loadLevelByUser=function(ent_code,items,callback){
    var openIds = [];
    _.forEach(items, function(d, i) {
        if(d.uid){
            openIds.push(d.uid);
        }
    });
    var tasks = [];
    _.forEach(openIds, function(open_id) {
        tasks.push(function(cont){
            loadUserLevel.loadLevelFromAPI(ent_code,open_id,cont);
        });
    });
    async.parallel(tasks,function(err,results){
        if (results && results.length>0) {//results: [{'egwegweg':1},{'gergergerg':3}]
            _.forEach(items, function(d, i) {
                if (items[i].toObject) {
                    items[i] = items[i].toObject();
                }
                _.forEach(results, function(r) {
                    if(r){
                        if(d.uid && r[d.uid]){
                            items[i].exp = r[d.uid];
                        }
                    }
                });

                if(!items[i].exp){
                    items[i].exp = 0;
                }
            });
        }
        return callback && callback();
    });
};


function loadLevelFromAPI(ent_code,open_id,callback){
    if(!ent_code || !open_id){
        return callback && callback(null,null);
    }
    var key = 'pisns-forum-thread:' +ent_code+':'+ open_id + ':exp';

    redis.get(key,function(err,value){
        if (value) {
            var result = JSON.parse(value);
            return callback && callback(null,result || null);
        } else {
            var url = API_ADDRESS + '/v1.0/internal/member/exp/byopenid?openId='+open_id+'&entCode='+ent_code;
            request.get({
                url: url,
                json: {}
            }, function(e, r, body) {
                if (e) {
                    console.log(e)
                }
                if(body && body.data){
                    redis.set(key, JSON.stringify(body.data));
                    redis.expire(key,2 * 60 );
                }
                return callback && callback(null,(body && body.data) || null);
            });
        }
    });
};